

#local
#code VCC_BYTE
#include "vars8.s"


; ________________________________
; get value from unsigned byte variable
;
; PEEKu8 ( &b -- n )

 macro PEEKu8
	ex	hl,de		; hl -> var
	ld	e,(hl)
	ld	d,0
 endm	

; ________________________________
; get value from signed byte variable
;
; PEEKs8 ( &b -- n )

 macro PEEKs8
	ld	a,(de)
	ld	e,a
 	add	a,a		; => nc || cy
 	sbc	a,a		; => 0 || $ff
 	ld	d,a		; de = lvar
 endm	

; ________________________________
; store value in byte variable
;
; POKE8 ( n &b -- )

 macro POKE8
 	ex	hl,de		; hl -> var
	pop	de
	ld	(hl),e
	pop	de
 endm

; ________________________________
; store value in byte variable
;
; NPOKE8 ( &b n -- )

 macro NPOKE8
	pop	hl		; hl -> var
	ld	(hl),e
	pop	de
 endm	


; ________________________________
; increment byte variable
;
; PPSET8 ( &b -- )

 macro PPSET8
	ex	hl,de		; hl->var
	inc	(hl)
	pop	de
 endm	

; ________________________________
; decrement byte variable
;
; MMSET8 ( &b -- )

 macro MMSET8
	ex	hl,de		; hl->var
	dec	(hl)
	pop	de
 endm

; ________________________________
; get value from unsigned byte variable
; with post-increment
;
; PEEKPPu8 ( &b -- n )

 macro PEEKPPu8
	ex	de,hl		; hl->var
	ld	e,(hl)
	inc	(hl)
	ld	d,0
 endm	

; ________________________________
; get value from unsigned byte variable
; with pre-decrement
;
; PEEKMMu8 ( &b -- n )

 macro PEEKMMu8
	ex	de,hl		; hl->var
	ld	e,(hl)
	dec	(hl)
	ld	d,0
 endm

; ________________________________
; get value from unsigned byte variable
; with pre-decrement
;
; MMPEEKu8 ( &b -- n )

 macro MMPEEKu8
	ex	de,hl		; hl->var
	dec	(hl)
	ld	e,(hl)
	ld	d,0
 endm	

; ________________________________
; get value from unsigned byte variable
; with post-increment
;
; PPPEEKu8 ( &b -- n )

 macro PPPEEKu8
	ex	de,hl		; hl->var
	inc	(hl)
	ld	e,(hl)
	ld	d,0
 endm	


; ________________________________
; get value from signed byte variable
; with post-increment
;
; PEEKPPs8 ( &b -- n )

 macro PEEKPPs8
 	call	peekpps8
 endm
 	
peekpps8::
	ex	de,hl		; hl->var
	ld	a,(hl)
	inc	(hl)

	ld	e,a
 	add	a,a		; => nc || cy
 	sbc	a,a		; => 0 || $ff
 	ld	d,a		; de = lvar
	ret

; ________________________________
; get value from signed byte variable
; with post-decrement
;
; PEEKMMs8 ( &b -- n )

 macro PEEKMMs8
 	call	peekmms8
 endm

peekmms8:: 
	ex	de,hl		; hl->var
	ld	a,(hl)
	dec	(hl)

	ld	e,a
 	add	a,a		; => nc || cy
 	sbc	a,a		; => 0 || $ff
 	ld	d,a		; de = lvar
	ret

; ________________________________
; get value from signed byte variable
; with pre-decrement
;
; MMPEEKs8 ( &b -- n )

 macro MMPEEKs8
 	call	mmpeeks8
 endm

mmpeeks8:: 
	ld	a,(de)
	dec	a
	ld	(de),a

	ld	e,a
 	add	a,a		; => nc || cy
 	sbc	a,a		; => 0 || $ff
 	ld	d,a		; de = lvar
	ret

; ________________________________
; get value from signed byte variable
; with pre-increment
;
; PPPEEKs8 ( &b -- n )

 macro PPPEEKs8
 	call	pppeeks8
 endm
 
pppeeks8::
	ld	a,(de)
	inc	a
	ld	(de),a

	ld	e,a
 	add	a,a		; => nc || cy
 	sbc	a,a		; => 0 || $ff
 	ld	d,a		; de = lvar
	ret




; ================================
;	    Arithmetics
; ================================
;
; note: values on stack are always 16 bit.
; only implementations for arithmetics on variables 
; need implementations here:

; ________________________________
; add value to byte variable
;
; ADDGL8 ( n &b -- )

 macro ADDGL8
 	pop	hl
	ld	a,(de)
	add	l
	ld	(de),a
	pop	de
 endm

; ________________________________
; subtract value from byte var.
;
; SUBGL8 ( n &b -- )

 macro SUBGL8
 	pop	hl
	ld	a,(de)
	sub	l
	ld	(de),a
	pop	de
 endm

; ________________________________
; reset bits in byte variable
;
; ANDGL8 ( n &b -- )

 macro ANDGL8
 	pop	hl
	ld	a,(de)
	and	l
	ld	(de),a
	pop	de
 endm

; ________________________________
; set bits in byte variable
;
; ORGL8 ( n &b -- )

 macro ORGL8
 	pop	hl
	ld	a,(de)
	or	l
	ld	(de),a
	pop	de
 endm

; ________________________________
; toggle bits in byte variable
;
; XORGL8 ( n &b -- )

 macro XORGL8
 	pop	hl
	ld	a,(de)
	xor	l
	ld	(de),a
	pop	de
 endm

; ________________________________
; shift left byte variable
;
; SLGL8 ( n &var -- )

 macro SLGL8
 	pop	hl
	call	slgl8
 endm
 
slgl8::				; hl = shift, de->var
 	ld	a,l
 	and	7
 	jr	z,2$
 	ld	l,a		; l = shift

	ld	a,(de)		; a = var	
1$:	add	a,a
	dec	l
	jr	nz,1$
	ld	(de),a		

2$:	pop	hl		; ret addr
	pop	de		; new top
	jp	hl

; ________________________________
; shift right unsigned byte variable
;
; SRGLu8 ( n &var -- )

 macro SRGLu8 
	pop	hl
	call	srglu8
 endm

srglu8::			; hl = shift, de -> var
 	ld	a,l
 	and	7
 	jr	z,2$
 	ld	l,a		; e = shift

	ld	a,(de)		; a = var	
1$:	srl	a
	dec	l
	jr	nz,1$	
	ld	(de),a

2$:	pop	hl		; ret addr
	pop	de		; new top
	jp	hl

; ________________________________
; shift right signed byte variable
;
; SRGLs8 ( n &var -- )

 macro SRGLs8
	pop	hl
	call 	srgls8
 endm

srgls8::			; hl = shift, de -> var
 	ld	a,l
 	and	7
 	jr	z,2$
 	ld	l,a		; e = shift

	ld	a,(de)		; a = var	
1$:	sra	a
	dec	l
	jr	nz,1$	
	ld	(de),a

2$:	pop	hl		; ret addr
	pop	de		; new top
	jp	hl



; ================================
;    Multiply, Divide, Remainder
; ================================

; ________________________________
; multiply byte variable
;
; MULGL8 ( n &b -- )

 macro MULGL8
 	pop	hl
	call	mulgl8
 endm

mulgl8:: 			; de->var, l=byte
	ex	hl,de		; hl->var, e=byte	
	push	hl		; stack:&var
	
	ld	a,(hl)		; a = var
	call	axel		; HL = L<<8 + A * DE
	ld	e,l

pop_hl_store_e_and_drop:
	pop	hl		; &var
	ld	(hl),e
	pop	hl		; ret addr
	pop	de		; new top
	jp	hl

; ________________________________
; divide unsigned byte variable
;
; DIVGLu8 ( n &b -- )

 macro DIVGLu8
 	pop	hl
	call	divglu8
 endm
 
divglu8::
	ex	hl,de		; hl -> var, de = value
	push	hl		; &var

	ld	l,(hl)		; l = var
	ld	h,0		; hl = var
	call	divu		; DE = HL / DE

	jr	pop_hl_store_e_and_drop
	
; ________________________________
; divide unsigned byte variable and store remainder
;
; REMGLu8 ( n &b -- )

 macro REMGLu8
 	pop	hl
	call	remglu8
 endm
 
remglu8::
	ex	hl,de		; hl -> var, de = value
	push	hl
	
	ld	l,(hl)		; l = var
	ld	h,0		; hl = var
	call	divu		; HL = HL % DE
	ex	hl,de
	
	jr	pop_hl_store_e_and_drop
	
; ________________________________
; divide signed byte variable
;
; DIVGLs8 ( n &b -- )

 macro DIVGLs8
 	pop	hl
	call	divgls8
 endm
 
divgls8::
	ex	hl,de		; hl -> var, de = value
	push	hl

	ld	a,(hl)		; a = var	
	ld	l,a		; l = var
	add	a,a		; cy=vz
	sbc	a,a		; 0x00 / 0xff
	ld	h,a		; hl = var

	call	divs		; DE = HL / DE

	jr	pop_hl_store_e_and_drop
	
; ________________________________
; divide signed byte variable and store remainder
;
; REMGLs8 ( n &b -- )

 macro REMGLs8
 	pop	hl
	call	remgls8
 endm

remgls8:: 
 	ex	hl,de		; hl -> var, de = value
	push	hl

	ld	a,(hl)		; a = var	
	ld	l,a
	add	a,a
	sbc	a,a
	ld	h,a
	call	rems		; HL = HL % DE
	ex	hl,de
	
	jr	pop_hl_store_e_and_drop


; ________________________________
; swap two byte variables
;
; SWAPVARS8 ( &v1 &v2 -- )

 macro SWAPVARS8
 	pop	hl
	call	swapvars8
 endm

swapvars8::
	ld	a,(de)
	ldi
	inc	bc	; undo
	dec	hl	; undo
	ld	(hl),a
	ret


; ________________________________
; classify char
;
; ISLETTER ( n -- f )

 macro ISLETTER
 	call	isletter
 endm

isletter::
	rlc	d
	ld	a,e
	ld	de,0
	ret	nz		; false: n.hi != 0 

	bit	6,a
	ret	z		; false: bit 0x40 must be set
	or	a,0x20		; map UC -> lc
	bit	7,a
	jr	nz,1$		; $E0..$FF

; $60..$7F:	
	sub	'a'
	cp	'z' - 'a' + 1
	ret	nc		; false
	inc	e
	ret			; true
	
1$:	cp	$F7		; '÷'
	ret	z		; false
	inc	e
	ret			; true
	
; ________________________________
; classify char
;
; ISBINDIGIT ( n -- f )

 macro ISBINDIGIT
 	call	isbindigit
 endm
 	
isbindigit::
	rlc	d
	ld	a,e
	ld	de,0	
	ret	nz		; false: n.hi != 0

	or	1
	cp	'1'
	ret	nz		; false
	inc	e
	ret			; true
	

; ________________________________
; classify char
;
; ISDECDIGIT ( n -- f )

 macro ISDECDIGIT
 	call	isdecdigit
 endm
 	
isdecdigit::
	rlc	d
	ld	a,e
	ld	de,0	
	ret	nz		; false: n.hi != 0

isdd:	sub	'0'
	cp	'9'-'0'+1
	ret	nc		; false
	inc	e
	ret			; true

; ________________________________
; classify char
;
; ISHEXDIGIT ( n -- f )

 macro ISHEXDIGIT
 	call	ishexdigit
 endm
 
ishexdigit::
	rlc	d
	ld	a,e
	ld	de,0
	ret	nz		; false: n.hi != 0

	cp	'A'
	jr	c,isdd		; 0..9

; A-F
	or	$20		; a-f
	sub	'a'
	cp	'f'-'a'+1
	ret	nc		; false
	inc	e
	ret			; true


; ________________________________
; calculate digit value
; no error checking
; up to base 36
; returned value ≥ base => error
; returned value < base => OK
;
; DIGITVALUE ( n -- n )

 macro DIGITVALUE
 	call	digitvalue
 endm
 
digitvalue::
	ld	a,e
	dec	a	; => 'A' starts at multiple of 0x20 => '@' and '`' result in ill. value
	cp	'9' + 1 - 1
	jr	c,1$		; '0'..'9'
	or	$20		; 'a'..'z'
	sub	'a' - '0' - 10 

1$:	sub	'0' - 1
	ld	e,a
extend_unsigned::
	ld	d,0
	ret

	

#endlocal




